# Generated by Django 3.2.18 on 2025-07-23 20:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('openlxp_xia', '0009_auto_20250225_1536'),
    ]

    operations = [
        migrations.AlterField(
            model_name='metadatafieldoverwrite',
            name='field_type',
            field=models.CharField(choices=[('datetime', 'DATETIME'), ('int', 'INTEGER'), ('str', 'CHARACTER'), ('URI', 'CHARACTER'), ('bool', 'BOOLEAN')], max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name='xisconfiguration',
            name='xis_api_key',
            field=models.CharField(help_text='Enter the XIS API Key', max_length=512),
        ),
    ]
