# Generated by Django 3.2.18 on 2025-08-04 20:15

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('openlxp_xia', '0013_supplementaltransmissionstatus'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='supplementalledger',
            name='supplemental_metadata_transmission_date',
        ),
        migrations.RemoveField(
            model_name='supplementalledger',
            name='supplemental_metadata_transmission_status',
        ),
        migrations.RemoveField(
            model_name='supplementalledger',
            name='supplemental_metadata_transmission_status_code',
        ),
    ]
