# Generated by Django 3.2.18 on 2025-07-31 17:54

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    dependencies = [
        ('openlxp_xia', '0011_xisconfiguration_publisher'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='metadataledger',
            name='target_metadata_transmission_date',
        ),
        migrations.RemoveField(
            model_name='metadataledger',
            name='target_metadata_transmission_status',
        ),
        migrations.RemoveField(
            model_name='metadataledger',
            name='target_metadata_transmission_status_code',
        ),
        migrations.CreateModel(
            name='metadataTransmissionStatus',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('target_metadata_transmission_status', models.CharField(blank=True, choices=[('Successful', 'S'), ('Failed', 'F'), ('Pending', 'P'), ('Ready', 'R')], default='Ready', max_length=20)),
                ('target_metadata_transmission_status_code', models.IntegerField(blank=True, null=True)),
                ('target_metadata_transmission_date', models.DateTimeField(blank=True, null=True)),
                ('XISConfiguration', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='openlxp_xia.xisconfiguration')),
                ('metadata_record', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='openlxp_xia.metadataledger')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
