# Generated by Django 3.2.18 on 2025-07-29 19:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('openlxp_xia', '0010_auto_20250723_2044'),
    ]

    operations = [
        migrations.AddField(
            model_name='xisconfiguration',
            name='publisher',
            field=models.CharField(default='default', help_text='Enter the publisher name', max_length=200),
            preserve_default=False,
        ),
    ]
